const {
  time,
  loadFixture,
} = require('@nomicfoundation/hardhat-toolbox/network-helpers');
const { expect } = require('chai');
const { ethers } = require('hardhat');

describe('Forum', () => {
  async function deploy() {
    const [account1, account2] = await ethers.getSigners();
    const DAO = await ethers.getContractFactory('DAO');
    const dao = await DAO.deploy();
    return { dao, account1, account2 };
  }
  let dao;
  let account1;
  let account2;
  const POOL_DURATION = 3600; // 1 hour
  const POOL_FEE = 100;
  const emptyCallbackData = ethers.AbiCoder.defaultAbiCoder().encode([], []);

  const initiateValidationPool = ({
    postIndex, duration,
    quorum, winRatio, bindingPercent,
    redistributeLosingStakes, callbackOnValidate,
    callbackData, fee,
  } = {}) => dao.initiateValidationPool(
    postIndex ?? 0,
    duration ?? POOL_DURATION,
    quorum ?? [1, 3],
    winRatio ?? [1, 2],
    bindingPercent ?? 100,
    redistributeLosingStakes ?? true,
    callbackOnValidate ?? false,
    callbackData ?? emptyCallbackData,
    { value: fee ?? POOL_FEE },
  );

  describe('Post', () => {
    beforeEach(async () => {
      ({ dao, account1, account2 } = await loadFixture(deploy));
    });

    it('should be able to add a post', async () => {
      const contentId = 'some-id';
      await expect(dao.addPost(account1, contentId, [])).to.emit(dao, 'PostAdded').withArgs(0);
      const post = await dao.posts(0);
      expect(post.author).to.equal(account1);
      expect(post.sender).to.equal(account1);
      expect(post.contentId).to.equal(contentId);
    });

    it('should be able to add a post on behalf of another account', async () => {
      const contentId = 'some-id';
      await dao.addPost(account2, contentId, []);
      const post = await dao.posts(0);
      expect(post.author).to.equal(account2);
      expect(post.sender).to.equal(account1);
      expect(post.contentId).to.equal(contentId);
    });

    it('should propagate reputation to citations', async () => {
      await dao.addPost(account1, 'content-id', []);
      await dao.addPost(account2, 'second-content-id', [{ weightPercent: 50, targetPostIndex: 0 }]);
      await initiateValidationPool({ postIndex: 1 });
      const pool = await dao.validationPools(0);
      expect(pool.postIndex).to.equal(1);
      await dao.evaluateOutcome(0);
      expect(await dao.balanceOf(account2)).to.equal(50);
      expect(await dao.balanceOf(account1)).to.equal(50);
    });
  });
});
